from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_indexing', '0003_auto_20150708_0101'),
    ]

    operations = [
        migrations.AlterField(
            model_name='index',
            name='label',
            field=models.CharField(
                unique=True, max_length=128, verbose_name='Label'
            ),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import (
    APIEventListView, APIEventTypeListView, APIEventTypeNamespaceDetailView,
    APIEventTypeNamespaceEventTypeListView, APIEventTypeNamespaceListView,
    APINotificationListView, APIObjectEventListView
)
from .views import (
    CurrentUserEventListView, EventListView, EventTypeSubscriptionListView,
    NotificationListView, NotificationMarkRead, NotificationMarkReadAll,
    ObjectEventListView, ObjectEventTypeSubscriptionListView,
    VerbEventListView
)

urlpatterns_events = [
    url(regex=r'^events/$', view=EventListView.as_view(), name='events_list'),
    url(
        regex=r'^object/(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/events/$',
        view=ObjectEventListView.as_view(), name='events_for_object'
    ),
    url(
        regex=r'^verbs/(?P<verb>[\w\-\.]+)/$',
        view=VerbEventListView.as_view(), name='events_by_verb'
    ),
    url(
        regex=r'^user/events/$', name='current_user_events',
        view=CurrentUserEventListView.as_view()
    ),
]

urlpatterns_notification = [
    url(
        regex=r'^user/notifications/$', view=NotificationListView.as_view(),
        name='user_notifications_list'
    ),
    url(
        regex=r'^user/notifications/(?P<pk>\d+)/mark_read/$',
        view=NotificationMarkRead.as_view(), name='notification_mark_read'
    ),
    url(
        regex=r'^user/notifications/all/mark_read/$',
        view=NotificationMarkReadAll.as_view(), name='notification_mark_read_all'
    ),
]

urlpatterns_subscriptions = [
    url(
        regex=r'^user/event_types/subscriptions/$',
        view=EventTypeSubscriptionListView.as_view(),
        name='event_types_user_subcriptions_list'
    ),
    url(
        regex=r'^user/object/(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/subscriptions/$',
        view=ObjectEventTypeSubscriptionListView.as_view(),
        name='object_event_types_user_subcriptions_list'
    ),
]

urlpatterns = []
urlpatterns.extend(urlpatterns_events)
urlpatterns.extend(urlpatterns_notification)
urlpatterns.extend(urlpatterns_subscriptions)

api_urls = [
    url(
        regex=r'^event_type_namespaces/(?P<name>[-\w]+)/$',
        view=APIEventTypeNamespaceDetailView.as_view(),
        name='event-type-namespace-detail'
    ),
    url(
        regex=r'^event_type_namespaces/(?P<name>[-\w]+)/event_types/$',
        view=APIEventTypeNamespaceEventTypeListView.as_view(),
        name='event-type-namespace-event-type-list'
    ),
    url(
        regex=r'^event_type_namespaces/$',
        view=APIEventTypeNamespaceListView.as_view(),
        name='event-type-namespace-list'
    ),
    url(
        regex=r'^event_types/$', view=APIEventTypeListView.as_view(),
        name='event-type-list'
    ),
    url(regex=r'^events/$', view=APIEventListView.as_view(), name='event-list'),
    url(
        regex=r'^notifications/$', view=APINotificationListView.as_view(),
        name='notification-list'
    ),
    url(
        regex=r'^objects/(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/events/$',
        view=APIObjectEventListView.as_view(), name='object-event-list'
    ),
]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('linking', '0007_auto_20180823_2353'),
    ]

    operations = [
        migrations.AlterField(
            model_name='smartlink',
            name='document_types',
            field=models.ManyToManyField(
                related_name='smart_links', to='documents.DocumentType',
                verbose_name='Document types'
            ),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.queues import queue_tools
from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_medium

queue_signatures = CeleryQueue(label=_('Signatures'), name='signatures', worker=worker_medium)

queue_signatures.add_task_type(
    label=_('Verify key signatures'),
    dotted_path='mayan.apps.document_signatures.tasks.task_verify_key_signatures'
)
queue_signatures.add_task_type(
    label=_('Unverify key signatures'),
    dotted_path='mayan.apps.document_signatures.tasks.task_unverify_key_signatures'
)
queue_signatures.add_task_type(
    label=_('Verify document version'),
    dotted_path='mayan.apps.document_signatures.tasks.task_verify_document_version'
)

queue_tools.add_task_type(
    label=_('Verify missing embedded signature'),
    dotted_path='mayan.apps.document_signatures.tasks.task_verify_missing_embedded_signature'
)



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import (
    APIDocumentTypeWorkflowRuntimeProxyListView, APIWorkflowDocumentTypeList,
    APIWorkflowDocumentTypeView, APIWorkflowImageView,
    APIWorkflowInstanceListView, APIWorkflowInstanceView,
    APIWorkflowInstanceLogEntryListView, APIWorkflowRuntimeProxyListView,
    APIWorkflowStateListView, APIWorkflowStateView,
    APIWorkflowTransitionListView, APIWorkflowTransitionView, APIWorkflowView
)
from .views.workflow_instance_views import (
    WorkflowInstanceDetailView, WorkflowInstanceListView,
    WorkflowInstanceTransitionSelectView,
    WorkflowInstanceTransitionExecuteView
)
from .views.workflow_proxy_views import (
    WorkflowRuntimeProxyDocumentListView,
    WorkflowRuntimeProxyListView, WorkflowRuntimeProxyStateDocumentListView,
    WorkflowRuntimeProxyStateListView
)
from .views.workflow_template_views import (
    DocumentTypeWorkflowTemplatesView, ToolLaunchWorkflows,
    WorkflowTemplateCreateView, WorkflowTemplateDeleteView,
    WorkflowTemplateEditView, WorkflowTemplateLaunchView,
    WorkflowTemplateListView, WorkflowTemplatePreviewView,
    WorkflowTemplateDocumentTypesView
)
from .views.workflow_template_state_views import (
    WorkflowTemplateStateActionCreateView,
    WorkflowTemplateStateActionDeleteView, WorkflowTemplateStateActionEditView,
    WorkflowTemplateStateActionListView,
    WorkflowTemplateStateActionSelectionView, WorkflowTemplateStateCreateView,
    WorkflowTemplateStateDeleteView, WorkflowTemplateStateEditView,
    WorkflowTemplateStateListView
)
from .views.workflow_template_transition_views import (
    WorkflowTemplateTransitionCreateView, WorkflowTemplateTransitionDeleteView,
    WorkflowTemplateTransitionEditView, WorkflowTemplateTransitionListView,
    WorkflowTemplateTransitionTriggerEventListView,
    WorkflowTemplateTransitionFieldCreateView,
    WorkflowTemplateTransitionFieldDeleteView,
    WorkflowTemplateTransitionFieldEditView,
    WorkflowTemplateTransitionFieldListView
)

urlpatterns_workflow_instances = [
    url(
        regex=r'^documents/(?P<pk>\d+)/workflows/$',
        view=WorkflowInstanceListView.as_view(),
        name='workflow_instance_list'
    ),
    url(
        regex=r'^documents/workflows/(?P<pk>\d+)/$',
        view=WorkflowInstanceDetailView.as_view(),
        name='workflow_instance_detail'
    ),
    url(
        regex=r'^documents/workflows/(?P<pk>\d+)/transitions/select/$',
        view=WorkflowInstanceTransitionSelectView.as_view(),
        name='workflow_instance_transition_selection'
    ),
    url(
        regex=r'^documents/workflows/(?P<workflow_instance_pk>\d+)/transitions/(?P<workflow_transition_pk>\d+)/execute/$',
        view=WorkflowInstanceTransitionExecuteView.as_view(),
        name='workflow_instance_transition_execute'
    ),
]

urlpatterns_workflow_runtime_proxies = [
    url(
        regex=r'workflow_runtime_proxies/$',
        view=WorkflowRuntimeProxyListView.as_view(),
        name='workflow_runtime_proxy_list'
    ),
    url(
        regex=r'^workflow_runtime_proxies/(?P<pk>\d+)/documents/$',
        view=WorkflowRuntimeProxyDocumentListView.as_view(),
        name='workflow_runtime_proxy_document_list'
    ),
    url(
        regex=r'^workflow_runtime_proxies/(?P<pk>\d+)/states/$',
        view=WorkflowRuntimeProxyStateListView.as_view(),
        name='workflow_runtime_proxy_state_list'
    ),
    url(
        regex=r'^workflow_runtime_proxies/states/(?P<pk>\d+)/documents/$',
        view=WorkflowRuntimeProxyStateDocumentListView.as_view(),
        name='workflow_runtime_proxy_state_document_list'
    ),
]

urlpatterns_workflow_states = [
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/states/$',
        view=WorkflowTemplateStateListView.as_view(),
        name='workflow_template_state_list'
    ),
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/states/create/$',
        view=WorkflowTemplateStateCreateView.as_view(),
        name='workflow_template_state_create'
    ),
    url(
        regex=r'^workflow_templates/states/(?P<pk>\d+)/delete/$',
        view=WorkflowTemplateStateDeleteView.as_view(),
        name='workflow_template_state_delete'
    ),
    url(
        regex=r'^workflow_templates/states/(?P<pk>\d+)/edit/$',
        view=WorkflowTemplateStateEditView.as_view(),
        name='workflow_template_state_edit'
    ),
]

urlpatterns_workflow_state_actions = [
    url(
        regex=r'^workflow_templates/states/(?P<pk>\d+)/actions/$',
        view=WorkflowTemplateStateActionListView.as_view(),
        name='workflow_template_state_action_list'
    ),
    url(
        regex=r'^workflow_templates/states/(?P<pk>\d+)/actions/selection/$',
        view=WorkflowTemplateStateActionSelectionView.as_view(),
        name='workflow_template_state_action_selection'
    ),
    url(
        regex=r'^workflow_templates/states/(?P<pk>\d+)/actions/(?P<class_path>[a-zA-Z0-9_.]+)/create/$',
        view=WorkflowTemplateStateActionCreateView.as_view(),
        name='workflow_template_state_action_create'
    ),
    url(
        regex=r'^workflow_templates/states/actions/(?P<pk>\d+)/delete/$',
        view=WorkflowTemplateStateActionDeleteView.as_view(),
        name='workflow_template_state_action_delete'
    ),
    url(
        regex=r'^workflow_templates/states/actions/(?P<pk>\d+)/edit/$',
        view=WorkflowTemplateStateActionEditView.as_view(),
        name='workflow_template_state_action_edit'
    ),
]

urlpatterns_workflow_templates = [
    url(
        regex=r'^workflow_templates/$', view=WorkflowTemplateListView.as_view(),
        name='workflow_template_list'
    ),
    url(
        regex=r'^workflow_templates/create/$', view=WorkflowTemplateCreateView.as_view(),
        name='workflow_template_create'
    ),
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/delete/$',
        view=WorkflowTemplateDeleteView.as_view(), name='workflow_template_delete'
    ),
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/document_types/$',
        view=WorkflowTemplateDocumentTypesView.as_view(),
        name='workflow_template_document_types'
    ),
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/edit/$',
        view=WorkflowTemplateEditView.as_view(), name='workflow_template_edit'
    ),
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/launch/$',
        view=WorkflowTemplateLaunchView.as_view(),
        name='workflow_template_launch'
    ),
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/preview/$',
        view=WorkflowTemplatePreviewView.as_view(),
        name='workflow_template_preview'
    ),
    url(
        regex=r'^document_types/(?P<pk>\d+)/workflow_templates/$',
        view=DocumentTypeWorkflowTemplatesView.as_view(),
        name='document_type_workflow_templates'
    ),
]
urlpatterns_workflow_transitions = [
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/transitions/$',
        view=WorkflowTemplateTransitionListView.as_view(),
        name='workflow_template_transition_list'
    ),
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/transitions/create/$',
        view=WorkflowTemplateTransitionCreateView.as_view(),
        name='workflow_template_transition_create'
    ),
    url(
        regex=r'^workflow_templates/(?P<pk>\d+)/transitions/events/$',
        view=WorkflowTemplateTransitionTriggerEventListView.as_view(),
        name='workflow_template_transition_events'
    ),
    url(
        regex=r'^workflow_templates/transitions/(?P<pk>\d+)/delete/$',
        view=WorkflowTemplateTransitionDeleteView.as_view(),
        name='workflow_template_transition_delete'
    ),
    url(
        regex=r'^workflow_templates/transitions/(?P<pk>\d+)/edit/$',
        view=WorkflowTemplateTransitionEditView.as_view(),
        name='workflow_template_transition_edit'
    ),
]

urlpatterns_workflow_transition_fields = [
    url(
        regex=r'^workflow_templates/transitions/(?P<pk>\d+)/fields/create/$',
        view=WorkflowTemplateTransitionFieldCreateView.as_view(),
        name='workflow_template_transition_field_create'
    ),
    url(
        regex=r'^workflow_templates/transitions/(?P<pk>\d+)/fields/$',
        view=WorkflowTemplateTransitionFieldListView.as_view(),
        name='workflow_template_transition_field_list'
    ),
    url(
        regex=r'^workflow_templates/transitions/fields/(?P<pk>\d+)/delete/$',
        view=WorkflowTemplateTransitionFieldDeleteView.as_view(),
        name='workflow_template_transition_field_delete'
    ),
    url(
        regex=r'^workflow_templates/transitions/fields/(?P<pk>\d+)/edit/$',
        view=WorkflowTemplateTransitionFieldEditView.as_view(),
        name='workflow_template_transition_field_edit'
    ),
]

urlpatterns_tools = [
    url(
        regex=r'^tools/workflows/launch/$',
        view=ToolLaunchWorkflows.as_view(),
        name='tool_launch_workflows'
    ),
]

urlpatterns = []
urlpatterns.extend(urlpatterns_tools)
urlpatterns.extend(urlpatterns_workflow_instances)
urlpatterns.extend(urlpatterns_workflow_runtime_proxies)
urlpatterns.extend(urlpatterns_workflow_states)
urlpatterns.extend(urlpatterns_workflow_state_actions)
urlpatterns.extend(urlpatterns_workflow_templates)
urlpatterns.extend(urlpatterns_workflow_transitions)
urlpatterns.extend(urlpatterns_workflow_transition_fields)

api_urls = [
    url(
        regex=r'^workflows/$', view=APIWorkflowRuntimeProxyListView.as_view(),
        name='workflow-list'
    ),
    url(
        regex=r'^workflows/(?P<pk>[0-9]+)/$', view=APIWorkflowView.as_view(),
        name='workflow-detail'
    ),
    url(
        regex=r'^workflows/(?P<pk>[0-9]+)/document_types/$',
        view=APIWorkflowDocumentTypeList.as_view(),
        name='workflow-document-type-list'
    ),
    url(
        regex=r'^workflows/(?P<pk>[0-9]+)/document_types/(?P<document_type_pk>[0-9]+)/$',
        view=APIWorkflowDocumentTypeView.as_view(),
        name='workflow-document-type-detail'
    ),
    url(
        regex=r'^workflows/(?P<pk>\d+)/image/$',
        name='workflow-image', view=APIWorkflowImageView.as_view()
    ),
    url(
        regex=r'^workflows/(?P<pk>[0-9]+)/states/$',
        view=APIWorkflowStateListView.as_view(), name='workflowstate-list'
    ),
    url(
        regex=r'^workflows/(?P<pk>[0-9]+)/states/(?P<state_pk>[0-9]+)/$',
        view=APIWorkflowStateView.as_view(), name='workflowstate-detail'
    ),
    url(
        regex=r'^workflows/(?P<pk>[0-9]+)/transitions/$',
        view=APIWorkflowTransitionListView.as_view(),
        name='workflowtransition-list'
    ),
    url(
        regex=r'^workflows/(?P<pk>[0-9]+)/transitions/(?P<transition_pk>[0-9]+)/$',
        view=APIWorkflowTransitionView.as_view(),
        name='workflowtransition-detail'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/workflows/$',
        view=APIWorkflowInstanceListView.as_view(),
        name='workflowinstance-list'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/workflows/(?P<workflow_pk>[0-9]+)/$',
        view=APIWorkflowInstanceView.as_view(),
        name='workflowinstance-detail'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/workflows/(?P<workflow_pk>[0-9]+)/log_entries/$',
        view=APIWorkflowInstanceLogEntryListView.as_view(),
        name='workflowinstancelogentry-list'
    ),
    url(
        regex=r'^document_types/(?P<pk>[0-9]+)/workflows/$',
        view=APIDocumentTypeWorkflowRuntimeProxyListView.as_view(),
        name='documenttype-workflow-list'
    ),
]



from __future__ import unicode_literals

from django import forms

from mayan.apps.documents.models import Document
from mayan.apps.templating.fields import TemplateField

from .models import WebLink


class WebLinkForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(WebLinkForm, self).__init__(*args, **kwargs)
        self.fields['template'] = TemplateField(
            initial_help_text=self.fields['template'].help_text,
            label=self.fields['template'].label, model=Document,
            model_variable='document', required=True
        )

    class Meta:
        fields = ('label', 'template', 'enabled')
        model = WebLink



from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon

icon_document_version_all_signature_verify = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_signature_list = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_list = Icon(
    driver_name='fontawesome', symbol='file-signature'
)

icon_document_version_signature_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_document_version_signature_details = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_detached_create = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_download = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_document_version_signature_embedded_create = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_list = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_upload = Icon(
    driver_name='fontawesome', symbol='upload'
)



from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from solo.models import SingletonModel

from .managers import AutoAdminSingletonManager


class AutoAdminSingleton(SingletonModel):
    account = models.ForeignKey(
        blank=True, null=True, on_delete=models.CASCADE,
        to=settings.AUTH_USER_MODEL, verbose_name=_('Account'),
    )
    password = models.CharField(
        blank=True, max_length=128, null=True, verbose_name=_('Password')
    )
    password_hash = models.CharField(
        blank=True, max_length=128, null=True, verbose_name=_('Password hash')
    )

    objects = AutoAdminSingletonManager()

    class Meta:
        verbose_name = verbose_name_plural = _('Autoadmin properties')



from __future__ import absolute_import, unicode_literals

import logging

from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.generics import ConfirmView
from mayan.apps.common.mixins import ExternalObjectMixin

from ..icons import icon_duplicated_document_list
from ..models import Document, DuplicatedDocument
from ..permissions import permission_document_tools, permission_document_view
from ..tasks import task_scan_duplicates_all

from .document_views import DocumentListView

__all__ = (
    'DocumentDuplicatesListView', 'DuplicatedDocumentListView',
    'ScanDuplicatedDocuments'
)
logger = logging.getLogger(__name__)


class DocumentDuplicatesListView(ExternalObjectMixin, DocumentListView):
    external_object_class = Document
    external_object_permission = permission_document_view
    external_object_pk_url_kwarg = 'document_id'

    def get_extra_context(self):
        context = super(DocumentDuplicatesListView, self).get_extra_context()
        context.update(
            {
                'no_results_icon': icon_duplicated_document_list,
                'no_results_text': _(
                    'Only exact copies of this document will be shown in the '
                    'this list.'
                ),
                'no_results_title': _(
                    'There are no duplicates for this document'
                ),
                'object': self.external_object,
                'title': _(
                    'Duplicates for document: %s'
                ) % self.external_object,
            }
        )
        return context

    def get_source_queryset(self):
        try:
            return DuplicatedDocument.objects.get(
                document=self.external_object
            ).documents.all()
        except DuplicatedDocument.DoesNotExist:
            return Document.objects.none()


class DuplicatedDocumentListView(DocumentListView):
    def get_document_queryset(self):
        return DuplicatedDocument.objects.get_duplicated_documents()

    def get_extra_context(self):
        context = super(DuplicatedDocumentListView, self).get_extra_context()
        context.update(
            {
                'no_results_icon': icon_duplicated_document_list,
                'no_results_text': _(
                    'Duplicates are documents that are composed of the exact '
                    'same file, down to the last byte. Files that have the '
                    'same text or OCR but are not identical or were saved '
                    'using a different file format will not appear as '
                    'duplicates.'
                ),
                'no_results_title': _(
                    'There are no duplicated documents'
                ),
                'title': _('Duplicated documents')
            }
        )
        return context


class ScanDuplicatedDocuments(ConfirmView):
    extra_context = {
        'title': _('Scan for duplicated documents?')
    }
    view_permission = permission_document_tools

    def view_action(self):
        task_scan_duplicates_all.apply_async()
        messages.success(
            message=_('Duplicated document scan queued successfully.'),
            request=self.request
        )



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.metadata.permissions import permission_document_metadata_edit
from mayan.apps.metadata.tests.mixins import MetadataTypeTestMixin

from ..permissions import (
    permission_group_create, permission_group_delete, permission_group_edit,
    permission_group_view, permission_user_create, permission_user_delete,
    permission_user_edit, permission_user_view
)

from .mixins import GroupTestMixin, GroupViewTestMixin, UserViewTestMixin


class GroupViewsTestCase(
    GroupTestMixin, GroupViewTestMixin, GenericViewTestCase
):
    def test_group_create_view_no_permission(self):
        group_count = Group.objects.count()

        response = self._request_test_group_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Group.objects.count(), group_count)

    def test_group_create_view_with_permission(self):
        self.grant_permission(permission=permission_group_create)

        group_count = Group.objects.count()

        response = self._request_test_group_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Group.objects.count(), group_count + 1)

    def test_group_delete_view_no_permission(self):
        self._create_test_group()

        group_count = Group.objects.count()

        response = self._request_test_group_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Group.objects.count(), group_count)

    def test_group_delete_view_with_access(self):
        self._create_test_group()
        self.grant_access(
            obj=self.test_group, permission=permission_group_delete
        )

        group_count = Group.objects.count()

        response = self._request_test_group_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Group.objects.count(), group_count - 1)

    def test_group_edit_view_no_permission(self):
        self._create_test_group()

        group_name = self.test_group.name

        response = self._request_test_group_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_group.refresh_from_db()
        self.assertEqual(self.test_group.name, group_name)

    def test_group_edit_view_with_access(self):
        self._create_test_group()
        self.grant_access(
            obj=self.test_group, permission=permission_group_edit
        )

        group_name = self.test_group.name

        response = self._request_test_group_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_group.refresh_from_db()
        self.assertNotEqual(self.test_group.name, group_name)

    def test_group_list_view_no_permission(self):
        self._create_test_group()

        response = self._request_test_group_list_view()
        self.assertNotContains(
            response=response, text=self.test_group.name, status_code=200
        )

    def test_group_list_view_with_permission(self):
        self._create_test_group()

        self.grant_access(
            obj=self.test_group, permission=permission_group_view
        )
        response = self._request_test_group_list_view()
        self.assertContains(
            response=response, text=self.test_group.name, status_code=200
        )

    def test_group_members_view_no_permission(self):
        self._create_test_user()
        self._create_test_group()
        self.test_user.groups.add(self.test_group)

        response = self._request_test_group_members_view()
        self.assertEqual(response.status_code, 404)

    def test_group_members_view_with_group_access(self):
        self._create_test_user()
        self._create_test_group()
        self.test_user.groups.add(self.test_group)
        self.grant_access(
            obj=self.test_group, permission=permission_group_edit
        )

        response = self._request_test_group_members_view()
        self.assertContains(
            response=response, text=self.test_group.name, status_code=200
        )
        self.assertNotContains(
            response=response, text=self.test_user.username, status_code=200
        )

    def test_group_members_view_with_user_access(self):
        self._create_test_user()
        self._create_test_group()
        self.test_user.groups.add(self.test_group)
        self.grant_access(obj=self.test_user, permission=permission_user_edit)

        response = self._request_test_group_members_view()
        self.assertNotContains(
            response=response, text=self.test_group.name, status_code=404
        )

    def test_group_members_view_with_full_access(self):
        self._create_test_user()
        self._create_test_group()
        self.test_user.groups.add(self.test_group)
        self.grant_access(
            obj=self.test_group, permission=permission_group_edit
        )
        self.grant_access(obj=self.test_user, permission=permission_user_edit)

        response = self._request_test_group_members_view()
        self.assertContains(
            response=response, text=self.test_user.username, status_code=200
        )
        self.assertContains(
            response=response, text=self.test_group.name, status_code=200
        )


class SuperUserViewTestCase(UserViewTestMixin, GenericViewTestCase):
    def setUp(self):
        super(SuperUserViewTestCase, self).setUp()
        self._create_test_superuser()

    def test_superuser_delete_view_with_access(self):
        superuser_count = get_user_model().objects.filter(
            is_superuser=True
        ).count()
        self.grant_access(
            obj=self.test_superuser, permission=permission_user_delete
        )
        response = self._request_test_superuser_delete_view()
        self.assertEqual(response.status_code, 404)
        self.assertEqual(
            get_user_model().objects.filter(is_superuser=True).count(),
            superuser_count
        )

    def test_superuser_detail_view_with_access(self):
        self.grant_access(
            obj=self.test_superuser, permission=permission_user_view
        )
        response = self._request_test_superuser_detail_view()
        self.assertEqual(response.status_code, 404)

    def test_superuser_normal_user_detail_view_with_access(self):
        self.grant_access(
            obj=self.test_superuser, permission=permission_user_view
        )

        self.test_user = self.test_superuser
        response = self._request_test_user_detail_view()
        self.assertEqual(response.status_code, 404)


class UserViewTestCase(UserViewTestMixin, GenericViewTestCase):
    def test_user_create_view_no_permission(self):
        user_count = get_user_model().objects.count()

        response = self._request_test_user_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(get_user_model().objects.count(), user_count)

    def test_user_create_view_with_permission(self):
        self.grant_permission(permission=permission_user_create)

        user_count = get_user_model().objects.count()

        response = self._request_test_user_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(get_user_model().objects.count(), user_count + 1)

    def test_user_delete_view_no_access(self):
        self._create_test_user()

        user_count = get_user_model().objects.count()

        response = self._request_test_user_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(get_user_model().objects.count(), user_count)

    def test_user_delete_view_with_access(self):
        self._create_test_user()

        user_count = get_user_model().objects.count()

        self.grant_access(obj=self.test_user, permission=permission_user_delete)

        response = self._request_test_user_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(get_user_model().objects.count(), user_count - 1)

    def test_user_multiple_delete_view_no_access(self):
        self._create_test_user()

        user_count = get_user_model().objects.count()

        response = self._request_test_user_delete_multiple_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(get_user_model().objects.count(), user_count)

    def test_user_multiple_delete_view_with_access(self):
        self._create_test_user()

        user_count = get_user_model().objects.count()

        self.grant_access(obj=self.test_user, permission=permission_user_delete)

        response = self._request_test_user_delete_multiple_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(get_user_model().objects.count(), user_count - 1)


class UserGroupViewTestCase(
    GroupTestMixin, UserViewTestMixin, GenericViewTestCase
):
    def test_user_groups_view_no_permission(self):
        self._create_test_user()
        self._create_test_group()
        self.test_user.groups.add(self.test_group)

        response = self._request_test_user_groups_view()
        self.assertEqual(response.status_code, 404)

    def test_user_groups_view_with_group_access(self):
        self._create_test_user()
        self._create_test_group()
        self.test_user.groups.add(self.test_group)
        self.grant_access(
            obj=self.test_group, permission=permission_group_edit
        )

        response = self._request_test_user_groups_view()
        self.assertNotContains(
            response=response, text=self.test_user.username, status_code=404
        )

    def test_user_groups_view_with_user_access(self):
        self._create_test_user()
        self._create_test_group()
        self.test_user.groups.add(self.test_group)
        self.grant_access(obj=self.test_user, permission=permission_user_edit)

        response = self._request_test_user_groups_view()
        self.assertContains(
            response=response, text=self.test_user.username, status_code=200
        )
        self.assertNotContains(
            response=response, text=self.test_group.name, status_code=200
        )

    def test_user_groups_view_with_full_access(self):
        self._create_test_user()
        self._create_test_group()
        self.test_user.groups.add(self.test_group)
        self.grant_access(
            obj=self.test_group, permission=permission_group_edit
        )
        self.grant_access(obj=self.test_user, permission=permission_user_edit)

        response = self._request_test_user_groups_view()
        self.assertContains(
            response=response, text=self.test_user.username, status_code=200
        )
        self.assertContains(
            response=response, text=self.test_group.name, status_code=200
        )


class MetadataLookupIntegrationTestCase(
    MetadataTypeTestMixin, GenericDocumentViewTestCase
):
    def setUp(self):
        super(MetadataLookupIntegrationTestCase, self).setUp()
        self._create_test_metadata_type()
        self.test_document_type.metadata.create(
            metadata_type=self.test_metadata_type
        )

    def test_user_list_lookup_render(self):
        self.test_metadata_type.lookup = '{{ users }}'
        self.test_metadata_type.save()
        self.test_document.metadata.create(
            metadata_type=self.test_metadata_type
        )

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self.get(
            viewname='metadata:metadata_edit', kwargs={
                'pk': self.test_document.pk
            }
        )
        self.assertContains(
            response=response, text='<option value="{}">{}</option>'.format(
                self._test_case_user.username, self._test_case_user.username
            ), status_code=200
        )

    def test_group_list_lookup_render(self):
        self.test_metadata_type.lookup = '{{ groups }}'
        self.test_metadata_type.save()
        self.test_document.metadata.create(
            metadata_type=self.test_metadata_type
        )
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self.get(
            viewname='metadata:metadata_edit', kwargs={
                'pk': self.test_document.pk
            }
        )

        self.assertContains(
            response=response, text='<option value="{}">{}</option>'.format(
                self._test_case_group.name, self._test_case_group.name
            ), status_code=200
        )



from __future__ import unicode_literals

import os
import sys

from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.utils import SettingNamespaceSingleton

from .literals import DEFAULT_SECRET_KEY, SECRET_KEY_FILENAME, SYSTEM_DIR

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

setting_namespace = SettingNamespaceSingleton(global_symbol_table=globals())
if 'revertsettings' in sys.argv:
    setting_namespace.update_globals(only_critical=True)
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(MEDIA_ROOT, 'db.sqlite3'),  # NOQA: F821
        }
    }
else:
    setting_namespace.update_globals()

# SECURITY WARNING: keep the secret key used in production secret!
environment_secret_key = os.environ.get('MAYAN_SECRET_KEY')
if environment_secret_key:
    SECRET_KEY = environment_secret_key
else:
    try:
        with open(os.path.join(MEDIA_ROOT, SYSTEM_DIR, SECRET_KEY_FILENAME)) as file_object:  # NOQA: F821
            SECRET_KEY = file_object.read().strip()
    except IOError:
        SECRET_KEY = DEFAULT_SECRET_KEY

# SECURITY WARNING: don't run with debug turned on in production!

# Application definition

INSTALLED_APPS = (
    # Placed at the top so it can override any template
    'mayan.apps.appearance',
    # Django
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.forms',
    # Allow using WhiteNoise in development
    'whitenoise.runserver_nostatic',
    'django.contrib.staticfiles',
    # 3rd party
    'actstream',
    'colorful',
    'corsheaders',
    'django_celery_beat',
    'formtools',
    'mathfilters',
    'mptt',
    'pure_pagination',
    'rest_framework',
    'rest_framework.authtoken',
    'solo',
    'stronghold',
    'widget_tweaks',
    # Base apps
    'mayan.apps.acls',
    'mayan.apps.authentication',
    'mayan.apps.autoadmin',
    'mayan.apps.common',
    'mayan.apps.converter',
    'mayan.apps.dashboards',
    'mayan.apps.dependencies',
    'mayan.apps.django_gpg',
    'mayan.apps.dynamic_search',
    'mayan.apps.events',
    'mayan.apps.file_caching',
    'mayan.apps.lock_manager',
    'mayan.apps.mimetype',
    'mayan.apps.navigation',
    'mayan.apps.permissions',
    'mayan.apps.platform',
    'mayan.apps.rest_api',
    'mayan.apps.smart_settings',
    'mayan.apps.task_manager',
    'mayan.apps.templating',
    'mayan.apps.user_management',
    # Project apps
    'mayan.apps.motd',
    # Document apps
    'mayan.apps.cabinets',
    'mayan.apps.checkouts',
    'mayan.apps.document_comments',
    'mayan.apps.document_indexing',
    'mayan.apps.document_parsing',
    'mayan.apps.document_signatures',
    'mayan.apps.document_states',
    'mayan.apps.documents',
    'mayan.apps.file_metadata',
    'mayan.apps.linking',
    'mayan.apps.mailer',
    'mayan.apps.mayan_statistics',
    'mayan.apps.metadata',
    'mayan.apps.mirroring',
    'mayan.apps.ocr',
    'mayan.apps.redactions',
    'mayan.apps.sources',
    'mayan.apps.storage',
    'mayan.apps.tags',
    'mayan.apps.web_links',
    # Placed after rest_api to allow template overriding
    'drf_yasg',
)

MIDDLEWARE = (
    'mayan.apps.common.middleware.error_logging.ErrorLoggingMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'whitenoise.middleware.WhiteNoiseMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'mayan.apps.common.middleware.timezone.TimezoneMiddleware',
    'stronghold.middleware.LoginRequiredMiddleware',
    'mayan.apps.common.middleware.ajax_redirect.AjaxRedirect',
)

ROOT_URLCONF = 'mayan.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader'
            ]
        },
    },
]

WSGI_APPLICATION = 'mayan.wsgi.application'

# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = '/static/'

# ------------ Custom settings section ----------

LANGUAGES = (
    ('ar', _('Arabic')),
    ('bg', _('Bulgarian')),
    ('bs', _('Bosnian')),
    ('cs', _('Czech')),
    ('da', _('Danish')),
    ('de', _('German')),
    ('el', _('Greek')),
    ('en', _('English')),
    ('es', _('Spanish')),
    ('fa', _('Persian')),
    ('fr', _('French')),
    ('hu', _('Hungarian')),
    ('id', _('Indonesian')),
    ('it', _('Italian')),
    ('lv', _('Latvian')),
    ('nl', _('Dutch')),
    ('pl', _('Polish')),
    ('pt', _('Portuguese')),
    ('pt-br', _('Portuguese (Brazil)')),
    ('ro', _('Romanian')),
    ('ru', _('Russian')),
    ('sl', _('Slovenian')),
    ('tr', _('Turkish')),
    ('vi', _('Vietnamese')),
    ('zh-hans', _('Chinese (Simplified)')),
)

SITE_ID = 1

STATIC_ROOT = os.environ.get(
    'MAYAN_STATIC_ROOT', os.path.join(MEDIA_ROOT, 'static')  # NOQA: F821
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

STATICFILES_STORAGE = 'whitenoise.storage.CompressedManifestStaticFilesStorage'

TEST_RUNNER = 'mayan.apps.common.tests.runner.MayanTestRunner'

# ---------- Django REST framework -----------

REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.TokenAuthentication',
        'rest_framework.authentication.BasicAuthentication',
    ),
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    'PAGE_SIZE': 10,
}

# --------- Pagination --------

PAGINATION_SETTINGS = {
    'PAGE_RANGE_DISPLAYED': 5,
    'MARGIN_PAGES_DISPLAYED': 2,
}

# ----------- Celery ----------

CELERY_ACCEPT_CONTENT = ('json',)
CELERY_BEAT_SCHEDULE = {}
CELERY_BEAT_SCHEDULER = 'django_celery_beat.schedulers.DatabaseScheduler'
CELERY_DISABLE_RATE_LIMITS = True
CELERY_ENABLE_UTC = True
CELERY_RESULT_SERIALIZER = 'json'
CELERY_TASK_ALWAYS_EAGER = False
CELERY_TASK_CREATE_MISSING_QUEUES = False
CELERY_TASK_EAGER_PROPAGATES = True
CELERY_TASK_QUEUES = []
CELERY_TASK_ROUTES = {}
CELERY_TASK_SERIALIZER = 'json'
CELERY_TIMEZONE = 'UTC'

# ------------ CORS ------------

CORS_ORIGIN_ALLOW_ALL = True

# ------ Timezone --------

TIMEZONE_COOKIE_NAME = 'django_timezone'
TIMEZONE_SESSION_KEY = 'django_timezone'

# ----- Stronghold -------

STRONGHOLD_PUBLIC_URLS = (r'^/docs/.+$', r'^/favicon\.ico$')

# ----- Swagger --------

SWAGGER_SETTINGS = {
    'DEFAULT_INFO': 'rest_api.schemas.openapi_info',
    'DEFAULT_MODEL_DEPTH': 1,
    'DOC_EXPANSION': 'None',
}

# ----- AJAX REDIRECT -----

AJAX_REDIRECT_CODE = 278

BASE_INSTALLED_APPS = INSTALLED_APPS

for app in INSTALLED_APPS:
    if 'mayan.apps.{}'.format(app) in BASE_INSTALLED_APPS:
        raise ImproperlyConfigured(
            'Update the app references in the file config.yml as detailed '
            'in https://docs.mayan-edms.com/releases/3.2.html#backward-incompatible-changes'
        )

for APP in (COMMON_EXTRA_APPS or ()):  # NOQA: F821
    INSTALLED_APPS = INSTALLED_APPS + (APP,)


INSTALLED_APPS = [
    APP for APP in INSTALLED_APPS if APP not in (COMMON_DISABLED_APPS or ())  # NOQA: F821
]

if not DATABASES:
    if DATABASE_ENGINE:  # NOQA: F821
        DATABASES = {
            'default': {
                'ENGINE': DATABASE_ENGINE,  # NOQA: F821
                'NAME': DATABASE_NAME,  # NOQA: F821
                'USER': DATABASE_USER,  # NOQA: F821
                'PASSWORD': DATABASE_PASSWORD,  # NOQA: F821
                'HOST': DATABASE_HOST,  # NOQA: F821
                'PORT': DATABASE_PORT,  # NOQA: F821
                'CONN_MAX_AGE': DATABASE_CONN_MAX_AGE  # NOQA: F821
            }
        }
    else:
        DATABASES = {
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': os.path.join(MEDIA_ROOT, 'db.sqlite3'),  # NOQA: F821
            }
        }



from __future__ import unicode_literals

from django import forms


class WorkflowImageWidget(forms.widgets.Widget):
    template_name = 'document_states/forms/widgets/workflow_image.html'

    def format_value(self, value):
        if value == '' or value is None:
            return None
        return value



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('Task manager'), name='task_manager')

permission_task_view = namespace.add_permission(
    label=_('View tasks'), name='task_view'
)



from __future__ import unicode_literals

import importlib
import logging

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.smart_settings.tests.mixins import SmartSettingTestMixin
from mayan.apps.sources import storages

from ..settings import setting_staging_file_image_cache_storage_arguments


class SourcesStorageSettingsTestCase(SmartSettingTestMixin, BaseTestCase):
    def test_setting_staging_file_image_cache_storage_arguments_invalid_value(self):
        self._set_environment_variable(
            name='MAYAN_{}'.format(
                setting_staging_file_image_cache_storage_arguments.global_name
            ), value="invalid_value"
        )
        self.test_case_silenced_logger_new_level = logging.FATAL + 10

        self._silence_logger(name='mayan.apps.sources.storages')

        with self.assertRaises(TypeError) as assertion:
            importlib.reload(storages)

        self.assertTrue('Unable to initialize' in str(assertion.exception))



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_indexing', '0009_auto_20150815_0351'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentIndexInstanceNode',
            fields=[
            ],
            options={
                'verbose_name': 'Document index node instance',
                'proxy': True,
                'verbose_name_plural': 'Document indexes node instances',
            },
            bases=('document_indexing.indexinstancenode',),
        ),
        migrations.CreateModel(
            name='IndexInstance',
            fields=[
            ],
            options={
                'verbose_name': 'Index instance',
                'proxy': True,
                'verbose_name_plural': 'Index instances',
            },
            bases=('document_indexing.index',),
        ),
    ]



from __future__ import unicode_literals

from django.test import override_settings

from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..classes import Template

TEST_TEMPLATE_RESULT = '<div'


class CommonAPITestCase(BaseAPITestCase):
    auto_login_user = False

    def _request_content_type_list_api_view(self):
        return self.get(viewname='rest_api:content-type-list')

    def test_content_type_list_api_view(self):
        response = self._request_content_type_list_api_view()
        self.assertEqual(response.status_code, 200)

    def test_template_detail_anonymous_view(self):
        template_main_menu = Template.get(name='menu_main')

        response = self.get(path=template_main_menu.get_absolute_url())
        self.assertNotContains(
            response=response, text=TEST_TEMPLATE_RESULT, status_code=403
        )

    @override_settings(LANGUAGE_CODE='de')
    def test_template_detail_view(self):
        self.login_user()
        template_main_menu = Template.get(name='menu_main')

        response = self.get(path=template_main_menu.get_absolute_url())
        self.assertContains(
            response=response, text=TEST_TEMPLATE_RESULT, status_code=200
        )



from __future__ import unicode_literals

from furl import furl

from django.urls import reverse

from mayan.apps.common.settings import setting_home_view
from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.documents.models import DocumentType
from mayan.apps.documents.permissions import (
    permission_document_properties_edit, permission_document_type_edit,
    permission_document_view
)
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.documents.tests.literals import TEST_DOCUMENT_TYPE_2_LABEL

from ..models import MetadataType
from ..permissions import (
    permission_document_metadata_add, permission_document_metadata_remove,
    permission_document_metadata_edit, permission_document_metadata_view,
    permission_metadata_type_create, permission_metadata_type_delete,
    permission_metadata_type_edit, permission_metadata_type_view
)

from .literals import (
    TEST_DOCUMENT_METADATA_VALUE_2, TEST_METADATA_TYPE_LABEL_2,
    TEST_METADATA_TYPE_NAME_2, TEST_METADATA_VALUE_EDITED
)
from .mixins import (
    DocumentMetadataViewTestMixin, MetadataTypeTestMixin,
    MetadataTypeViewTestMixin
)


class DocumentMetadataViewTestCase(
    DocumentMetadataViewTestMixin, MetadataTypeTestMixin,
    GenericDocumentViewTestCase
):
    def setUp(self):
        super(DocumentMetadataViewTestCase, self).setUp()
        self._create_test_metadata_type()
        self.test_document_type.metadata.create(
            metadata_type=self.test_metadata_type
        )

    def _create_test_document_metadata(self):
        self.test_document_metadata = self.test_document.metadata.create(
            metadata_type=self.test_metadata_type, value=''
        )

    def test_document_metadata_add_get_view_no_permission(self):
        response = self._request_test_document_metadata_add_get_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(self.test_document.metadata.count(), 0)

    def test_document_metadata_add_get_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_add
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_add
        )

        response = self._request_test_document_metadata_add_get_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(self.test_document.metadata.count(), 0)

    def test_document_metadata_add_post_view_no_permission(self):
        response = self._request_test_document_metadata_add_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(self.test_document.metadata.count(), 0)

    def test_document_metadata_add_post_view_with_document_access(self):
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_add
        )

        response = self._request_test_document_metadata_add_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(self.test_document.metadata.count(), 0)

    def test_document_metadata_add_post_view_with_metadata_type_access(self):
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_add
        )

        response = self._request_test_document_metadata_add_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(self.test_document.metadata.count(), 0)

    def test_document_metadata_add_post_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_add
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_add
        )

        response = self._request_test_document_metadata_add_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(self.test_document.metadata.count(), 1)

    def test_document_metadata_add_redirect(self):
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_add
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_add
        )

        response = self._request_test_document_metadata_add_post_view()

        self.assertRedirects(
            response=response, expected_url=reverse(
                viewname='metadata:metadata_edit', kwargs={
                    'pk': self.test_document.pk
                }
            ), status_code=302,
            target_status_code=404
        )

    def test_document_multiple_metadata_add_redirect(self):
        self.upload_document()

        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_metadata_add
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_metadata_add
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_add
        )

        response = self._request_test_document_multiple_metadata_add_post_view()

        url = furl(response.url)

        self.assertEqual(
            url.path, reverse(viewname='metadata:metadata_multiple_edit')
        )

        self.assertEqual(
            set(map(int, url.args['id_list'].split(','))),
            set([self.test_documents[0].pk, self.test_documents[1].pk])
        )

    def test_document_multiple_metadata_add_post_view_with_document_access(self):
        self.upload_document()

        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_metadata_add
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_metadata_add
        )

        response = self._request_test_document_multiple_metadata_add_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(self.test_documents[0].metadata.count(), 0)
        self.assertEqual(self.test_documents[1].metadata.count(), 0)

    def test_document_metadata_multiple_add_post_view_with_full_access(self):
        self._create_test_metadata_type()
        self.test_document_type.metadata.create(
            metadata_type=self.test_metadata_types[1]
        )

        self.grant_permission(permission=permission_document_view)
        self.grant_permission(permission=permission_document_metadata_add)
        self.grant_permission(permission=permission_document_metadata_edit)

        response = self._request_test_document_metadata_multiple_add_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_document.metadata.filter(
                metadata_type__in=self._get_test_metadata_type_queryset()
            ).count(), self._get_test_metadata_type_queryset().count()
        )

    def test_document_metadata_edit_post_view_no_permission(self):
        self._create_test_document_metadata()
        document_metadata_value = self.test_document.metadata.first().value
        response = self._request_test_document_metadata_edit_post_view()
        self.assertEqual(response.status_code, 404)

        self.test_document.metadata.first().refresh_from_db()

        self.assertEqual(
            self.test_document.metadata.first().value, document_metadata_value
        )

    def test_document_metadata_edit_post_view_with_document_access(self):
        self._create_test_document_metadata()
        document_metadata_value = self.test_document.metadata.first().value
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )

        response = self._request_test_document_metadata_edit_post_view()
        self.assertEqual(response.status_code, 302)

        self.test_document.metadata.first().refresh_from_db()

        self.assertEqual(
            self.test_document.metadata.first().value, document_metadata_value
        )

    def test_document_metadata_edit_post_view_with_metadata_type_access(self):
        self._create_test_document_metadata()
        document_metadata_value = self.test_document.metadata.first().value
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_test_document_metadata_edit_post_view()
        self.assertEqual(response.status_code, 404)

        self.test_document.metadata.first().refresh_from_db()

        self.assertEqual(
            self.test_document.metadata.first().value, document_metadata_value
        )

    def test_document_metadata_edit_post_view_with_full_access(self):
        self._create_test_document_metadata()
        document_metadata_value = self.test_document.metadata.first().value
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_test_document_metadata_edit_post_view()
        self.assertEqual(response.status_code, 302)

        self.test_document.metadata.first().refresh_from_db()

        self.assertNotEqual(
            self.test_document.metadata.first().value, document_metadata_value
        )

    def test_document_metadata_edit_after_document_type_change(self):
        # Gitlab issue #204
        # Problems to add required metadata after changing the document type

        self.grant_permission(permission=permission_document_properties_edit)
        self.grant_permission(permission=permission_document_metadata_edit)
        self.grant_permission(permission=permission_document_metadata_view)

        document_type_2 = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE_2_LABEL
        )

        metadata_type_2 = MetadataType.objects.create(
            name=TEST_METADATA_TYPE_NAME_2, label=TEST_METADATA_TYPE_LABEL_2
        )

        document_metadata_2 = document_type_2.metadata.create(
            metadata_type=metadata_type_2, required=True
        )

        self.test_document.set_document_type(document_type=document_type_2)

        response = self.get(
            viewname='metadata:metadata_edit', kwargs={
                'pk': self.test_document.pk
            }, follow=True
        )
        self.assertContains(response=response, text='Edit', status_code=200)

        response = self.post(
            'metadata:metadata_edit', kwargs={'pk': self.test_document.pk},
            data={
                'form-0-id': document_metadata_2.metadata_type.pk,
                'form-0-update': True,
                'form-0-value': TEST_DOCUMENT_METADATA_VALUE_2,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': '',
            }, follow=True
        )
        self.assertContains(
            response=response, text='Metadata for document', status_code=200
        )

        self.assertEqual(
            self.test_document.metadata.get(metadata_type=metadata_type_2).value,
            TEST_DOCUMENT_METADATA_VALUE_2
        )

    def test_document_metadata_edit_redirect(self):
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_test_document_metadata_edit_post_view()

        self.assertRedirects(
            response=response, expected_url=reverse(
                viewname='metadata:metadata_view', kwargs={
                    'pk': self.test_document.pk
                }
            ), status_code=302,
            target_status_code=404
        )

    def test_document_multiple_metadata_edit_redirect(self):
        self.upload_document()

        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_metadata_edit
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_edit
        )

        response = self._request_test_document_multiple_metadata_edit_post_view()
        self.assertEqual(response.status_code, 302)

        url = furl(response.url)
        self.assertEqual(
            url.path, reverse(viewname='metadata:metadata_multiple_edit')
        )

        self.assertEqual(
            set(map(int, url.args['id_list'].split(','))),
            set([self.test_documents[0].pk, self.test_documents[1].pk])
        )

    def test_document_metadata_list_view_no_access(self):
        self._create_test_document_metadata()

        response = self._request_test_document_metadata_list_view()

        self.assertNotContains(
            response=response, text=self.test_document.label,
            status_code=404
        )
        self.assertNotContains(
            response=response, text=self.test_metadata_type.label,
            status_code=404
        )

    def test_document_metadata_list_view_with_document_access(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_view
        )

        response = self._request_test_document_metadata_list_view()

        self.assertContains(
            response=response, text=self.test_document.label,
            status_code=200
        )
        self.assertNotContains(
            response=response, text=self.test_metadata_type.label,
            status_code=200
        )

    def test_document_metadata_list_view_with_metadata_type_access(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_view
        )

        response = self._request_test_document_metadata_list_view()

        self.assertNotContains(
            response=response, text=self.test_document.label,
            status_code=404
        )
        self.assertNotContains(
            response=response, text=self.test_metadata_type.label,
            status_code=404
        )

    def test_document_metadata_list_view_with_full_access(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_view
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_view
        )

        response = self._request_test_document_metadata_list_view()

        self.assertContains(
            response=response, text=self.test_document.label,
            status_code=200
        )
        self.assertContains(
            response=response, text=self.test_metadata_type.label,
            status_code=200
        )

    def test_document_metadata_remove_get_view_no_permission(self):
        self._create_test_document_metadata()

        response = self._request_test_document_metadata_remove_get_view()

        self.assertNotContains(
            response=response, text=self.test_metadata_type.label, status_code=404
        )
        self.assertTrue(
            self.test_document_metadata in self.test_document.metadata.all()
        )

    def test_document_metadata_remove_get_view_with_full_access(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_remove,
        )
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_remove,
        )

        # Silence unrelated logging
        self._silence_logger(name='mayan.apps.navigation.classes')

        response = self._request_test_document_metadata_remove_get_view()
        self.assertContains(
            response, text=self.test_metadata_type.label, status_code=200
        )
        self.assertTrue(
            self.test_document_metadata in self.test_document.metadata.all()
        )

    def test_document_metadata_remove_post_view_no_permission(self):
        self._create_test_document_metadata()

        response = self._request_test_document_metadata_remove_get_view()
        self.assertNotContains(
            response=response, text=self.test_metadata_type.label, status_code=404
        )

        self.assertTrue(
            self.test_document_metadata in self.test_document.metadata.all()
        )

    def test_document_metadata_remove_post_view_with_document_access(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_remove
        )
        response = self._request_test_document_metadata_remove_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self.test_document_metadata in self.test_document.metadata.all()
        )

    def test_document_metadata_remove_post_view_with_metadata_type_access(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_remove
        )
        response = self._request_test_document_metadata_remove_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self.test_document_metadata in self.test_document.metadata.all()
        )

    def test_document_metadata_remove_post_view_with_full_access(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_remove
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_remove
        )
        response = self._request_test_document_metadata_remove_post_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(
            self.test_document_metadata not in self.test_document.metadata.all()
        )

    def test_document_metadata_remove_redirect(self):
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_remove
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_remove
        )

        response = self._request_test_document_metadata_remove_post_view()

        self.assertRedirects(
            response=response, expected_url=reverse(
                viewname='metadata:metadata_view', kwargs={
                    'pk': self.test_document.pk
                }
            ), status_code=302,
            target_status_code=404
        )

    def test_document_multiple_metadata_remove_redirect(self):
        self.upload_document()

        self.test_documents[0].metadata.create(
            metadata_type=self.test_metadata_type
        )
        self.test_documents[1].metadata.create(
            metadata_type=self.test_metadata_type
        )

        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_metadata_remove
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_metadata_remove
        )
        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_document_metadata_remove
        )

        response = self._request_test_document_multiple_metadata_remove_post_view()
        self.assertRedirects(
            response=response, expected_url=reverse(
                viewname=setting_home_view.value
            ), status_code=302,
            target_status_code=200
        )

    def test_document_multiple_metadata_remove_view_with_access(self):
        self.grant_permission(permission=permission_document_view)
        self.grant_permission(permission=permission_document_metadata_remove)

        self.upload_document()

        document_metadata = self.test_documents[0].metadata.create(
            metadata_type=self.test_metadata_type
        )
        self.test_documents[1].metadata.create(
            metadata_type=self.test_metadata_type
        )

        response = self.get(
            viewname='metadata:metadata_multiple_remove', data={
                'id_list': '{},{}'.format(
                    self.test_documents[0].pk, self.test_documents[0].pk
                )
            }
        )
        self.assertEqual(response.status_code, 200)

        # Test post to metadata removal view
        response = self.post(
            viewname='metadata:metadata_multiple_remove', data={
                'id_list': '{},{}'.format(
                    self.test_documents[0].pk, self.test_documents[1].pk
                ),
                'form-0-id': document_metadata.metadata_type.pk,
                'form-0-update': True,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': '',
            }
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(self.test_documents[0].metadata.count(), 0)
        self.assertEqual(self.test_documents[1].metadata.count(), 0)

    def test_multiple_document_metadata_edit(self):
        self.upload_document()

        self.grant_permission(permission=permission_document_view)
        self.grant_permission(permission=permission_document_metadata_add)
        self.grant_permission(permission=permission_document_metadata_edit)

        document_metadata = self.test_documents[0].metadata.create(
            metadata_type=self.test_metadata_type
        )
        self.test_documents[1].metadata.create(
            metadata_type=self.test_metadata_type
        )

        response = self.get(
            viewname='metadata:metadata_multiple_edit', data={
                'id_list': '{},{}'.format(
                    self.test_documents[0].pk, self.test_documents[1].pk
                )
            }
        )
        self.assertContains(response=response, text='Edit', status_code=200)

        # Test post to metadata removal view
        response = self.post(
            viewname='metadata:metadata_multiple_edit', data={
                'id_list': '{},{}'.format(
                    self.test_documents[0].pk, self.test_documents[1].pk
                ),
                'form-0-id': document_metadata.metadata_type.pk,
                'form-0-value': TEST_METADATA_VALUE_EDITED,
                'form-0-update': True,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': '',
            }, follow=True
        )
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self.test_documents[0].metadata.first().value, TEST_METADATA_VALUE_EDITED
        )
        self.assertEqual(
            self.test_documents[1].metadata.first().value, TEST_METADATA_VALUE_EDITED
        )


class DocumentMetadataRequiredTestCase(
    DocumentMetadataViewTestMixin, MetadataTypeTestMixin, GenericDocumentViewTestCase
):
    def setUp(self):
        super(DocumentMetadataRequiredTestCase, self).setUp()
        self._create_test_metadata_type()
        self._create_test_metadata_type()
        self.test_document_type.metadata.create(
            metadata_type=self.test_metadata_types[0]
        )
        self.test_document_type.metadata.create(
            metadata_type=self.test_metadata_types[1], required=True
        )

    def _create_test_document_metadata(self):
        self.test_document_metadata = self.test_document.metadata.get_or_create(
            metadata_type=self.test_metadata_types[0]
        )
        self.test_document.metadata.get_or_create(
            metadata_type=self.test_metadata_types[1]
        )

    def test_document_metadata_remove_post_view_with_access_non_required_from_dual(self):
        # Adds two metadata types to the document type: one required and
        # one optional. Add both to a document. Attempts to remove the optional
        # one.
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_remove
        )
        self.grant_access(
            obj=self.test_metadata_types[0],
            permission=permission_document_metadata_remove
        )
        self.grant_access(
            obj=self.test_metadata_types[1],
            permission=permission_document_metadata_remove
        )

        response = self._request_test_document_metadata_remove_post_view(index=0)
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            self.test_document.metadata.filter(
                metadata_type=self.test_metadata_types[0]
            ).exists()
        )
        self.assertTrue(
            self.test_document.metadata.filter(
                metadata_type=self.test_metadata_types[1]
            ).exists()
        )

    def test_document_metadata_remove_post_view_with_access_required_from_dual(self):
        # Adds two metadata types to the document type: one required and
        # one optional. Add both to a document. Attempts to remove the required
        # one.
        self._create_test_document_metadata()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_metadata_remove
        )
        self.grant_access(
            obj=self.test_metadata_types[0],
            permission=permission_document_metadata_remove
        )
        self.grant_access(
            obj=self.test_metadata_types[1],
            permission=permission_document_metadata_remove
        )

        response = self._request_test_document_metadata_remove_post_view(index=1)
        self.assertEqual(response.status_code, 302)

        self.assertTrue(
            self.test_document.metadata.filter(
                metadata_type=self.test_metadata_types[0]
            ).exists()
        )
        self.assertTrue(
            self.test_document.metadata.filter(
                metadata_type=self.test_metadata_types[1]
            ).exists()
        )


class MetadataTypeViewTestCase(
    MetadataTypeViewTestMixin, MetadataTypeTestMixin, GenericViewTestCase
):

    def test_metadata_type_create_view_no_permission(self):
        metadata_type_count = MetadataType.objects.count()

        response = self._request_test_metadata_type_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(
            MetadataType.objects.count(), metadata_type_count
        )

    def test_metadata_type_create_view_with_access(self):
        self.grant_permission(permission=permission_metadata_type_create)
        metadata_type_count = MetadataType.objects.count()

        response = self._request_test_metadata_type_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            MetadataType.objects.count(), metadata_type_count + 1
        )

    def test_metadata_type_delete_view_no_permission(self):
        self._create_test_metadata_type()
        metadata_type_count = MetadataType.objects.count()

        response = self._request_test_metadata_type_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            MetadataType.objects.count(), metadata_type_count
        )

    def test_metadata_type_delete_view_with_access(self):
        self._create_test_metadata_type()

        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_metadata_type_delete
        )
        metadata_type_count = MetadataType.objects.count()

        response = self._request_test_metadata_type_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            MetadataType.objects.count(), metadata_type_count - 1
        )

    def test_metadata_type_edit_view_no_permission(self):
        self._create_test_metadata_type()
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self.test_metadata_type
        )

        response = self._request_test_metadata_type_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_metadata_type.refresh_from_db()
        self.assertEqual(
            self._model_instance_to_dictionary(
                instance=self.test_metadata_type
            ), metadata_type_values
        )

    def test_metadata_type_edit_view_with_access(self):
        self._create_test_metadata_type()

        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_metadata_type_edit
        )
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self.test_metadata_type
        )

        response = self._request_test_metadata_type_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_metadata_type.refresh_from_db()
        self.assertNotEqual(
            self._model_instance_to_dictionary(
                instance=self.test_metadata_type
            ), metadata_type_values
        )

    def test_metadata_type_list_view_no_permission(self):
        self._create_test_metadata_type()

        response = self._request_metadata_type_list_view()
        self.assertNotContains(
            response=response, text=self.test_metadata_type, status_code=200
        )

    def test_metadata_type_list_view_with_access(self):
        self._create_test_metadata_type()

        self.grant_access(
            obj=self.test_metadata_type,
            permission=permission_metadata_type_view
        )

        response = self._request_metadata_type_list_view()
        self.assertContains(
            response=response, text=self.test_metadata_type, status_code=200
        )


class MetadataTypeDocumentTypeViewTestCase(
    MetadataTypeViewTestMixin, MetadataTypeTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_document = False

    def test_metadata_type_relationship_view_no_permission(self):
        self._create_test_metadata_type()
        self.upload_document()

        response = self._request_test_metadata_type_relationship_edit_view()
        self.assertEqual(response.status_code, 403)

        self.test_document_type.refresh_from_db()
        self.assertEqual(self.test_document_type.metadata.count(), 0)

    def test_metadata_type_relationship_view_with_document_type_access(self):
        self._create_test_metadata_type()
        self.upload_document()

        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_edit
        )

        response = self._request_test_metadata_type_relationship_edit_view()
        self.assertEqual(response.status_code, 403)

        self.test_document_type.refresh_from_db()
        self.assertEqual(self.test_document_type.metadata.count(), 0)

    def test_metadata_type_relationship_view_with_metadata_type_access(self):
        self._create_test_metadata_type()
        self.upload_document()

        self.grant_access(
            obj=self.test_metadata_type, permission=permission_metadata_type_edit
        )

        response = self._request_test_metadata_type_relationship_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_document_type.refresh_from_db()
        self.assertEqual(self.test_document_type.metadata.count(), 0)

    def test_metadata_type_relationship_view_with_metadata_type_and_document_type_access(self):
        self._create_test_metadata_type()
        self.upload_document()

        self.grant_access(
            obj=self.test_metadata_type, permission=permission_metadata_type_edit
        )
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_edit
        )

        response = self._request_test_metadata_type_relationship_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_document_type.refresh_from_db()
        self.assertQuerysetEqual(
            qs=self.test_document_type.metadata.values('metadata_type', 'required'),
            values=[
                {
                    'metadata_type': self.test_metadata_type.pk,
                    'required': True,
                }
            ], transform=dict
        )



from __future__ import unicode_literals

import logging

from django.utils.six import raise_from

from mayan.apps.storage.utils import get_storage_subclass

from .settings import (
    setting_documentimagecache_storage,
    setting_documentimagecache_storage_arguments,
    setting_storage_backend, setting_storage_backend_arguments
)

logger = logging.getLogger(__name__)

try:
    storage_documentversion = get_storage_subclass(
        dotted_path=setting_storage_backend.value
    )(**setting_storage_backend_arguments.value)
except Exception as exception:
    message = (
        'Unable to initialize the document version storage. Check the '
        'settings {} and {} for formatting errors.'.format(
            setting_storage_backend.global_name,
            setting_storage_backend_arguments.global_name
        )
    )

    logger.fatal(message)
    raise_from(value=TypeError(message), from_value=exception)

try:
    storage_documentimagecache = get_storage_subclass(
        dotted_path=setting_documentimagecache_storage.value
    )(**setting_documentimagecache_storage_arguments.value)
except Exception as exception:
    message = (
        'Unable to initialize the document image cache storage. '
        'Check the settings {} and {} for formatting errors.'.format(
            setting_documentimagecache_storage.global_name,
            setting_documentimagecache_storage_arguments.global_name
        )
    )

    logger.fatal(message)
    raise_from(value=TypeError(message), from_value=exception)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0013_auto_20190725_0452'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userlocaleprofile',
            name='language',
            field=models.CharField(choices=[('ar', 'Arabic'), ('bg', 'Bulgarian'), ('bs', 'Bosnian'), ('cs', 'Czech'), ('da', 'Danish'), ('de', 'German'), ('el', 'Greek'), ('en', 'English'), ('es', 'Spanish'), ('fa', 'Persian'), ('fr', 'French'), ('hu', 'Hungarian'), ('id', 'Indonesian'), ('it', 'Italian'), ('lv', 'Latvian'), ('nl', 'Dutch'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Portuguese (Brazil)'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sl', 'Slovenian'), ('tr', 'Turkish'), ('vi', 'Vietnamese'), ('zh-hans', 'Chinese (Simplified)')], max_length=8, verbose_name='Language'),
        ),
    ]



from __future__ import absolute_import, unicode_literals

import logging

from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.django_gpg.models import Key
from mayan.apps.django_gpg.permissions import permission_key_sign
from mayan.apps.document_states.classes import WorkflowAction
from mayan.apps.document_states.exceptions import WorkflowStateActionError

from .models import DetachedSignature, EmbeddedSignature

logger = logging.getLogger(__name__)


class DocumentSignatureDetachedAction(WorkflowAction):
    fields = {
        'key': {
            'label': _('Key'),
            'class': 'django.forms.ModelChoiceField', 'kwargs': {
                'help_text': _(
                    'Private key that will be used to sign the document '
                    'version.'
                ), 'queryset': Key.objects.none(),
            },
        }, 'passphrase': {
            'label': _('Passphrase'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'The passphrase to unlock the key and allow it to be '
                    'used to sign the document version.'
                ), 'required': False
            },
        },
    }
    field_order = ('key', 'passphrase')
    label = _('Sign document (detached)')
    widgets = {
        'passphrase': {
            'class': 'django.forms.widgets.PasswordInput',
        }
    }

    def get_arguments(self, context):
        latest_version = context['document'].latest_version
        if not latest_version:
            raise WorkflowStateActionError(
                _(
                    'Document has no version to sign. You might be trying to '
                    'use this action in an initial state before the created '
                    'document is yet to be processed.'
                )
            )

        return {
            'document_version': latest_version,
            'key': Key.objects.get(pk=self.form_data['key']),
            'passphrase': self.form_data.get('passphrase')
        }

    def get_form_schema(self, request):
        user = request.user
        logger.debug('user: %s', user)

        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_key_sign, queryset=Key.objects.all(),
            user=user
        )

        self.fields['key']['kwargs']['queryset'] = queryset
        return super(DocumentSignatureDetachedAction, self).get_form_schema(
            request=request
        )

    def execute(self, context):
        DetachedSignature.objects.sign_document_version(
            **self.get_arguments(context=context)
        )


class DocumentSignatureEmbeddedAction(DocumentSignatureDetachedAction):
    label = _('Sign document (embedded)')

    def execute(self, context):
        EmbeddedSignature.objects.sign_document_version(
            **self.get_arguments(context=context)
        )



from __future__ import unicode_literals

from django.db import models, transaction
from django.template import Context, Template
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.events import event_document_type_edited
from mayan.apps.documents.models import DocumentType

from .events import event_web_link_created, event_web_link_edited
from .managers import WebLinkManager


@python_2_unicode_compatible
class WebLink(models.Model):
    """
    This model stores the basic fields for a web link. Web links allow
    generating links from documents to external resources.
    """
    label = models.CharField(
        db_index=True, help_text=_('A short text describing the web link.'),
        max_length=96, verbose_name=_('Label')
    )
    template = models.TextField(
        help_text=_(
            'Template that will be used to craft the final URL of the '
            'web link.'
        ), verbose_name=_('Template')
    )
    enabled = models.BooleanField(default=True, verbose_name=_('Enabled'))
    document_types = models.ManyToManyField(
        related_name='web_links', to=DocumentType,
        verbose_name=_('Document types')
    )

    objects = WebLinkManager()

    class Meta:
        ordering = ('label',)
        verbose_name = _('Web link')
        verbose_name_plural = _('Web links')

    def __str__(self):
        return self.label

    def document_types_add(self, queryset, _user=None):
        with transaction.atomic():
            event_web_link_edited.commit(
                actor=_user, target=self
            )
            for obj in queryset:
                self.document_types.add(obj)
                event_document_type_edited.commit(
                    actor=_user, action_object=self, target=obj
                )

    def document_types_remove(self, queryset, _user=None):
        with transaction.atomic():
            event_web_link_edited.commit(
                actor=_user, target=self
            )
            for obj in queryset:
                self.document_types.remove(obj)
                event_document_type_edited.commit(
                    actor=_user, action_object=self, target=obj
                )

    def get_absolute_url(self):
        return reverse(
            viewname='weblinks:web_link_edit', kwargs={
                'pk': self.pk
            }
        )

    def save(self, *args, **kwargs):
        _user = kwargs.pop('_user', None)

        with transaction.atomic():
            is_new = not self.pk
            super(WebLink, self).save(*args, **kwargs)
            if is_new:
                event_web_link_created.commit(
                    actor=_user, target=self
                )
            else:
                event_web_link_edited.commit(
                    actor=_user, target=self
                )


class ResolvedWebLink(WebLink):
    """
    Proxy model to represent an already resolved web link. Used for easier
    colums registration.
    """
    class Meta:
        proxy = True

    def get_url_for(self, document):
        context = Context({'document': document})

        return Template(self.template).render(context=context)



from __future__ import unicode_literals

import importlib
import logging

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents import storages
from mayan.apps.documents.settings import (
    setting_documentimagecache_storage_arguments,
    setting_storage_backend_arguments
)
from mayan.apps.smart_settings.tests.mixins import SmartSettingTestMixin

from ..settings import setting_language_codes


class DocumentSettingsTestCase(SmartSettingTestMixin, BaseTestCase):
    def test_documents_language_codes_setting_double_quotes(self):
        self._set_environment_variable(
            name='MAYAN_{}'.format(setting_language_codes.global_name),
            value='["spa","fra"]'
        )

        self.assertEqual(
            setting_language_codes.value,
            ['spa', 'fra']
        )

    def test_documents_language_codes_setting_single_quotes(self):
        self._set_environment_variable(
            name='MAYAN_{}'.format(setting_language_codes.global_name),
            value="['spa','deu']"
        )

        self.assertEqual(
            setting_language_codes.value,
            ['spa', 'deu']
        )


class DocumentStorageSettingsTestCase(SmartSettingTestMixin, BaseTestCase):
    def test_setting_documentimagecache_storage_arguments_invalid_value(self):
        self._set_environment_variable(
            name='MAYAN_{}'.format(
                setting_documentimagecache_storage_arguments.global_name
            ), value="invalid_value"
        )
        self.test_case_silenced_logger_new_level = logging.FATAL + 10

        self._silence_logger(name='mayan.apps.documents.storages')

        with self.assertRaises(TypeError) as assertion:
            importlib.reload(storages)

        self.assertTrue('Unable to initialize' in str(assertion.exception))

    def test_setting_storage_backend_arguments_invalid_value(self):
        self._set_environment_variable(
            name='MAYAN_{}'.format(
                setting_storage_backend_arguments.global_name
            ), value="invalid_value"
        )
        self.test_case_silenced_logger_new_level = logging.FATAL + 10

        self._silence_logger(name='mayan.apps.documents.storages')

        with self.assertRaises(TypeError) as assertion:
            importlib.reload(storages)

        self.assertTrue('Unable to initialize' in str(assertion.exception))



from __future__ import unicode_literals

import json

from mayan.apps.django_gpg.tests.literals import TEST_KEY_PRIVATE_PASSPHRASE
from mayan.apps.django_gpg.tests.mixins import KeyTestMixin
from mayan.apps.document_states.literals import WORKFLOW_ACTION_ON_ENTRY
from mayan.apps.document_states.tests.mixins import WorkflowTestMixin
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..models import DetachedSignature, EmbeddedSignature
from ..workflow_actions import (
    DocumentSignatureDetachedAction, DocumentSignatureEmbeddedAction
)


class DocumentSignatureWorkflowActionTestCase(
    GenericDocumentViewTestCase, KeyTestMixin, WorkflowTestMixin,
):
    auto_upload_document = False

    def test_document_signature_detached_action(self):
        self.upload_document()
        self._create_test_key_private()
        signature_count = DetachedSignature.objects.count()

        action = DocumentSignatureDetachedAction(
            form_data={
                'key': self.test_key_private.pk,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )
        action.execute(context={'document': self.test_document})
        self.assertNotEqual(signature_count, DetachedSignature.objects.count())

    def test_document_signature_embedded_action(self):
        self.upload_document()
        self._create_test_key_private()
        signature_count = EmbeddedSignature.objects.count()

        action = DocumentSignatureEmbeddedAction(
            form_data={
                'key': self.test_key_private.pk,
                'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
            }
        )
        action.execute(context={'document': self.test_document})
        self.assertNotEqual(signature_count, EmbeddedSignature.objects.count())

    def test_document_signature_detached_action_via_workflow(self):
        self._create_test_workflow()
        self.test_workflow.document_types.add(self.test_document_type)
        self._create_test_workflow_states()
        self._create_test_workflow_transitions()
        self._create_test_key_private()

        self.test_workflow_state_2.actions.create(
            label='test action', when=WORKFLOW_ACTION_ON_ENTRY,
            enabled=True,
            action_path='mayan.apps.document_signatures.workflow_actions.DocumentSignatureDetachedAction',
            action_data=json.dumps(
                {
                    'key': self.test_key_private.pk,
                    'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
                }
            ),
        )

        self.upload_document()
        self.test_workflow_instance = self.test_document.workflows.first()

        signature_count = DetachedSignature.objects.count()

        self.test_workflow_instance.do_transition(
            transition=self.test_workflow_transition
        )
        self.assertNotEqual(signature_count, DetachedSignature.objects.count())

    def test_document_signature_embedded_action_via_workflow(self):
        self._create_test_workflow()
        self.test_workflow.document_types.add(self.test_document_type)
        self._create_test_workflow_states()
        self._create_test_workflow_transitions()
        self._create_test_key_private()

        self.test_workflow_state_2.actions.create(
            label='test action', when=WORKFLOW_ACTION_ON_ENTRY,
            enabled=True,
            action_path='mayan.apps.document_signatures.workflow_actions.DocumentSignatureEmbeddedAction',
            action_data=json.dumps(
                {
                    'key': self.test_key_private.pk,
                    'passphrase': TEST_KEY_PRIVATE_PASSPHRASE
                }
            ),
        )

        self.upload_document()
        self.test_workflow_instance = self.test_document.workflows.first()

        signature_count = EmbeddedSignature.objects.count()

        self.test_workflow_instance.do_transition(
            transition=self.test_workflow_transition
        )
        self.assertNotEqual(signature_count, EmbeddedSignature.objects.count())
